//
//  mcCMController.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-07.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>
#import "mcCMModel.h"

@class mcMusiCogController, mcCMModel, mcWMStream, mcWMSegment, mcCMAssociation, mcCMMotive;

enum mcLearnMode {
    pitchMode,
    rhythmMode,
    pitchAndRhythmMode
};
typedef enum mcLearnMode mcLearnMode;

enum mcInvarianceType {
    interval,
    scale_step,
    frequencyRatio,
    icVector
};
typedef enum mcInvarianceType mcInvarianceType;

enum mcIdentityType {
    pitch,
    pitchClass,
    frequency
};
typedef enum mcIdentityType mcIdentityType;

enum mcRInvarianceType {
    beatED,
    IOIRatio
};
typedef enum mcRInvarianceType mcRInvarianceType;

enum mcRIdentityType {
    IOIinBeats,
    IOIinMS
};
typedef enum mcRIdentityType mcRIdentityType;

enum mcConnectionType {
    childConnection,
    parentConnection,
    targetConnection,
    terminalConnection
};
typedef enum mcConnectionType mcConnectionType;

@interface mcCMController : NSObject

@property (weak, nonatomic) mcMusiCogController* musiCog;               // reference to the main controller
@property (strong, nonatomic) mcCMModel* pitchModel;                    // the pitch CM
@property (strong, nonatomic) mcCMModel* rhythmModel;                   // the rhythm CM
@property (assign, nonatomic) mcLearnMode learnMode;                    // "pitchMode", "rhythmMode", "pitchAndRhythmMode"
@property (assign, nonatomic) BOOL isStart;                             // yes if the last input was the first EVER input
@property (assign, nonatomic) BOOL learnFromFeedback;                   // yes if we should learn from previous generated output
@property (assign, nonatomic) BOOL anythingLearned;                     // yes if the CM has some content
@property (assign, nonatomic) NSUInteger topLevel;                      // the highest level learned during training
@property (assign, nonatomic) enum mcInvarianceType invarianceType;     // use to set the invariance type for the pitchModel
@property (assign, nonatomic) enum mcIdentityType identityType;         // use to set the identity type for the pitchModel
@property (assign, nonatomic) enum mcRInvarianceType rInvarianceType;   // invariance type for rhythmModel
@property (assign, nonatomic) enum mcRIdentityType rIdentityType;       // identity type for rhythmModel
@property (strong, nonatomic) NSPointerArray* associativeMemory;        // sparse 2D array: node->ID : {Assoc 1, ..., Assoc n} <-- also sparse

@property (assign, nonatomic) NSUInteger memoryResetCount;

- (void) resetState;
- (UInt64) totalConnectionsLearned;
- (mcWMSegment*) searchForParallelismInStream:(mcWMStream *)aStream givenInputChunk :(mcWMSegment *)aChunk;
- (void) processNoteSegment:(mcWMSegment *)aSegment fromStream:(mcWMStream *)aStream willLearn:(BOOL)flag;
- (void) processContiguousChunks:(id)contigChunks fromStream:(mcWMStream*)aStream;
- (mcCMEdge*) extendLevelFromNode:(mcCMNode*)aNode withEvent:(mcEvent*)anEvent;
- (mcCMEdge*) addSuperTierToNode:(mcCMNode*)aNode forEvent:(mcEvent*)anEvent withParentNode:(mcCMNode*)parentNode;
- (float) segment:(mcWMSegment*)aSegment canPredictInput:(mcEvent*)anInput;
- (BOOL) updateBoundaryEventOfChunk:(mcWMSegment*)aChunk inStream:(mcWMStream*)aStream;
- (mcWMSegment*) buildNewChunkForRecognizedSegment:(mcWMSegment*)aSegment inStream:(mcWMStream*)aStream;
- (void) ensureInvarianceEdgeForIdentityEdgeFromParent:(mcCMNode *)aParent toChild:(mcCMNode *)aChild;
- (void) ensureInvarianceEdgeForIdentityEdgeFromBoundary:(mcCMNode *)aBoundary toTerminal:(mcCMNode *)aTerminal;
- (void) ensureInvarianceEdgeForIdentityEdgeFromTerminal:(mcCMNode*)aTerminal toBoundary:(mcCMNode*)aBoundary;
- (void) pruneAcrossTierConnections;
- (void) createLinkForChunk:(mcWMSegment*)aChunk withInference:(mcCMNode*)anInference;
- (void) createLinkForPreviousChunk:(mcWMSegment *)prevChunk withCurrentInference:(mcCMNode *)inference;
- (void) createLinkFromSubLevelTerminal:(mcCMNode *)slTerminal toLevelBoundary:(mcCMNode *)levelBoundary;
- (void) ensureInvarianceOrHigherTerminalSetForGeneration:(NSMutableArray *)nodeEdges givenBoundary:(mcCMNode *)boundaryNode;
- (NSMutableArray*) contextSensitiveProbabilisticSegmentFromTerminalEdgeSet:(NSMutableArray *)terminalEdges
                                                             withGoalMotive:(mcCMMotive *)aMotive;
- (NSMutableArray*) contextSensitiveRhythmicSegmentFromTerminalEdgeSet:(NSMutableArray*)terminalEdges withGoalMotive:(mcCMMotive*)aMotive;
- (NSMutableArray*) probabilisticRhythmicSegmentForGoalMotive:(mcCMMotive*)aMotive;
- (NSMutableArray*) getRhythmicSegmentFromWMStream:(mcWMStream*)aStream forGoalMotive:(mcCMMotive*)aMotive;
- (NSMutableArray*) extractRhythmPathForSegment:(mcWMSegment*)aSegment;
- (float) getDurationOfRhythmModelPath:(NSArray*)rPath;
- (void) getAllPaths:(NSMutableArray*)paths toLowerDepthNode:(mcCMNode*)aNode visitedNodesBuffer:(NSMutableArray*)visited;
- (mcCMEdge*) weightedProbabilisticChoiceFromNodeEdgeArray:(NSArray *)nodeEdges
                                      includeFutureEntropy:(BOOL)useEntropy
                                withFutureDistributionType:(enum mcConnectionType)connectionType;
- (mcCMEdge*) weightedProbabilisticChoiceOfSuperTierNodeEdgeUsingParentChildWeightFromEdgeSet:(NSArray*)edgeSet;
- (NSArray*) generateOrdinalFormWithLength:(UInt16)length atLevel:(UInt16)levelNum;
- (NSArray*) getWithinLevelParentEdgesToNode:(mcCMNode*)aNode;
- (NSArray*) getInvariancePathForSegment:(mcWMSegment*)aSegment fromModel:(mcCMModel*)aModel;
- (mcCMEdge*) contextSensitiveSelectionOfParentForNode:(mcCMNode *)aChild fromParentEdgeSet:(NSArray *)parentEdges;
- (NSMutableArray*) findHighProbabilityPathToTerminal:(mcCMNode *)aTerminal;
- (NSArray*) findTerminalEdgeSetByTunnelingOnLevelNumber:(uint16)levelNum inModel:(mcCMModel *)mModel;
- (mcCMNode*) findTerminalWithRelationToBoundary:(mcCMNode*)aBoundary;
- (mcCMNode*) findBoundaryWithRelationToState:(mcCMState*)aState onLevelNumber:(NSUInteger)levelNum;
- (mcCMEdge*) findParentEdgeForOutNode:(mcCMNode*)outNode;
- (mcCMAssociation*) weightedProbabilisticChoiceFromAssociationArray:(NSMutableArray *)associations;
- (NSArray*) extractOrdinalPhraseStructureForLevel:(UInt16)levelNum;
- (void) addAssociation:(mcCMAssociation*)anAssoc;
- (mcCMEdge*) weightedProbabilisticChoiceFromNodeEdgeArray:(NSArray *)nodeEdges;
- (UInt64) getCountSumForEdgeBranch:(NSArray*)edgeBranch;
- (void) fillPitchBranch:(NSMutableArray *)pBranch andRhythmBranch:(NSMutableArray *)rBranch forPitchTerminal:(mcCMEdge *)pTerminal;
- (NSArray*) getBranchForNode:(mcCMNode *)aNode;
- (NSArray*) sortedRhythmicAssociationsForNode:(mcCMNode *)aNode;
- (UInt64) maxNodeCount;
- (UInt16) validateMotive:(mcCMMotive*)aMotive;
- (BOOL) validatePath:(NSArray*)aPath;
- (id) initWithMusiCog:(mcMusiCogController*)mc;

@end
