//
//  mcCMModel.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-07.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>

@class mcCMController, mcEvent, mcCMState, mcCMLevel, mcCMNode, mcCMEdge;

enum mcModelType {
    pitchType,
    rhythmType
};
typedef enum mcModelType mcModelType;

@interface mcCMModel : NSObject

@property (weak, nonatomic) mcCMController* controller;     // referece to the CM controller
@property (strong, nonatomic) NSMutableArray* nodeMap;      // all nodes added to this model (at all levels) - nodeMap[0] = "NULL"
@property (strong, nonatomic) NSHashTable* edgeMap;         // all parent->child edges (i.e., reciprocal -- NOT terminals, targets, superTiers)
@property (strong, nonatomic) NSPointerArray* adjacencyList;// 2d array of parent/child relationships: node index -> {(UInt64)edgeID 1, ..., edgeID n}
@property (strong, nonatomic) NSArray* levels;              // all levels - initialize to MAX_LEVELS
@property (strong, nonatomic) mcCMState* state;             // the currently active node across all levels
@property (assign, nonatomic) mcModelType type;             // "pitchType" or "rhythmType" (the enum is in mcMusiCogController)
@property (assign, nonatomic) float *invarianceBins;
@property (assign, nonatomic) long *invarianceWeights;
@property (assign, nonatomic) float *identityBins;
@property (assign, nonatomic) long *identityWeights;

- (mcCMLevel*) getLevel:(UInt16)levelNum;
- (id) initWithController:(mcCMController*)cmController;
- (UInt32) invarianceBinForValue:(float)aValue;
- (UInt32) identityBinForValue:(float)aValue;
- (void) normalizeWeights:(long *)weights toVector:(float *)normVector withLength:(long)length;
- (float) updateInvarianceGridWithEvent:(mcEvent *)anEvent;
- (float) updateIdentityGridWithEvent:(mcEvent*)anEvent;
- (UInt32) countNodesOnTier:(UInt8)tier;
- (void) clearModel;
- (void) initLevels;

@end

/*****
 *  NOTE: The arrays stored in the adjacencyList will be NSPointerArrays. This is only to allow them to be handled in the same
 *  manner as the sparse arrays in the mcCMLevel class (terminalNodes, targetNodes, and superTierNodes).
 **/
