//
//  mcMusiCogController.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-08.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

@protocol MCLogging <NSObject>

@required
- (void) postLogMessage:(NSString*)aString;

@end

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>
#import "mcEvent.h"

@class mcPerceptionModule, mcWMController, mcCMController, mcPMController, mcMusicPlayerController, MusiCogLog;

@interface mcMusiCogController : NSObject {
    void* musicog_external;
    NSOperationQueue* mcOperationQueue;
}

@property (weak, nonatomic) id<MCLogging> delegate;

@property (strong, nonatomic) NSOperationQueue* mcOperationQueue;
@property (strong, nonatomic) mcPerceptionModule* PE;           // the PE doesn't have any data, so it's basically a controller class anyway
@property (strong, nonatomic) mcWMController* WMController;
@property (strong, nonatomic) mcCMController* CMController;
@property (strong, nonatomic) mcPMController* PMController;
@property (strong, nonatomic) mcMusicPlayerController* musicPlayerController;
@property (strong, nonatomic) MusiCogLog* log;
@property (assign, nonatomic) BOOL isGenerating;
@property (assign, nonatomic) UInt64 totalConnections;
@property (strong, nonatomic) NSString* work;
@property (strong, nonatomic) NSString* corpus;
@property (strong, nonatomic) NSNumber* generationCounter;
@property (strong, nonatomic) NSNumber* numGenerations;
@property (strong, nonatomic) NSNumber* streamCapacity;
@property (strong, nonatomic) NSNumber* cohesionTolerance;
@property (strong, nonatomic) NSNumber* tempo;
@property (assign, nonatomic) BOOL randomizeTraining;

// Some extra analysis info
@property (assign, nonatomic) float L1SegmentCount;

// ADMM generation
@property (strong, nonatomic) NSNumber* admmEvents;
@property (strong, nonatomic) NSNumber* admmDuration;
@property (assign, nonatomic) BOOL autoExportMIDIFiles;
@property (strong, nonatomic) NSNumber* filesToGenerate;

// Max external support ----------------------------------
@property (assign, nonatomic) BOOL allowCallsToMax;
@property (assign, nonatomic) MCMusicEvent* maxOutEvent;

// Benching and debugging --------------------------------
@property (assign, nonatomic) BOOL logSegmentsToText;
@property (assign, nonatomic) double machTimerMSMultiplier;
@property (assign, nonatomic) NSUInteger totalNodes;
@property (assign, nonatomic) NSUInteger totalEvents;
@property (assign, nonatomic) NSUInteger prevTotal;
@property (retain, nonatomic) NSMutableArray* nodesForAllWorks;

- (void) clearPreviousGeneration;
- (void) clearPreviousGenerationMaintainingWM;
- (void) clearPreviousGenerationMaintainingWMAndSequence;
- (void) analyzeMIDIFilesAtPath:(NSString*)filePath;
- (void) analyzeMIDIFile;
- (void) runArbitraryTest;
- (BOOL) saveGeneratedPerformanceToMIDIFile;
- (BOOL) makeCorpusMIDIFilesLegato;
- (void) clearLTM;
- (void) toggleSequencer:(BOOL)toggleState;

- (void) generate;
- (void) generateContinuation;
- (void) logToMax:(NSString*)logMessage;
- (void) setMusicog_external:(void *)musicog_external;
- (void*) musicog_external;
- (void) sendOutputEvent;
- (void) finishBang;

- (void) ADMMGenerateEvents:(NSUInteger)length;
- (void) ADMMGenerateDuration:(float)duration;
- (void) saveADMMGeneratedSequenceToMIDIFile;

// Accessors for other modules
// PM
- (void) setPMBeatsInGenerationBar:(UInt8)beatsInBar;        // Beats in bar for generation
- (void) setPMBarsToGenerate:(UInt8)bars;                    // Number of bars to generate
- (void) setPMGenerateContinuously:(BOOL)flag;               // Whether to continue when plan runs out
- (void) setPMGenerateNonRealTime:(BOOL)flag;                // Whether to generate in "offline" mode
- (void) setPMFilesToGenerate:(UInt8)numFiles;
// PE
// NOTE: this is actually implemented in the MCWMStream (so, in WM, but PE is clearer)
// WM
- (void) setWMStreamCapacity:(UInt8)capacity;                // number of elements in a stream
// CM
- (void) setCMLearnMode:(UInt8)mode;                         // 0=pitch, 1=rhythm, 2=pitch+rhythm (3=harmony, 4=all) default = 2
- (void) setCMLearnFromInput:(BOOL)flag;
- (void) setLockToInferredModeAndTonality:(BOOL)flag;
//- (void) setGenerateHomophony:(BOOL)flag;
- (NSString*) getBuildDate;

@end

static const UInt8 MAX_LEVELS = 16;
static const UInt32 IDENTITY_SIZE = 256;
static const UInt32 INVARIANCE_SIZE = 256;
