//
//  mcPlanner.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-09.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>

@class mcMusiCogController, mcCMModel, mcCMMotive, mcWMStream, mcWMSegment, mcCMEdge;

enum mcPlanningStatus {
    plannerFailed,
    plannerSwitchToBottomUp,
    plannerFailedOnTerminalNode,
    plannerFailedOnInvalidBranch,
    plannerFailedOnInvalidMotive,
    plannerFailedOnNULLMotive,
    plannerFailedWithEmptyPlan,
    plannerSucceded
};
typedef enum mcPlanningStatus mcPlanningStatus;

enum mcMotiveValidationStatus {
    motiveInvalidPitchBoundary = 1001,
    motiveInvalidPitchTerminal = 1002,
    motiveInvalidRhythmBoundary = 2001,
    motiveInvalidRhythmTerminal = 2002,
    motiveInvalidPitchBranch = 1000,
    motiveInvalidRhythmBranch = 2000,
    motiveValidationSucceeded = 1
};
typedef enum mcMotiveValidationStatus mcMotiveValidationStatus;

@interface mcPlanner : NSObject

@property (weak, nonatomic) mcMusiCogController* musiCog;
@property (strong, nonatomic) NSMutableDictionary* planSet;
@property (strong, nonatomic) NSMutableDictionary* ordinalPlanMap;
/***
 This is a 2D dictionary where the outer key is the planSet level (e.g., "L2"), and the inner dictionary has to types of
 entry: 1) <key>"ordinalForm":<value>(NSArray){0,1,0}, and 2) <key>(NSNumber)Index:<value>(mcCMMotive)motive. The key
 "ordinalForm" holds the generated ordinal form array, as a record of the order of motives. The NSNumber keys are used
 to map indices from the ordinal form array to Schema tier mcCMMotive objects (which will represent the motives in the
 ordinal form).
 */

@property (assign, nonatomic) enum mcTier planningTier;
@property (strong, nonatomic) NSMutableDictionary* primedPhraseBoundaries;

- (enum mcPlanningStatus) createPlanSet;
- (void) createPlanSetForModel:(mcCMModel*)aModel;
- (NSMutableArray*) getPlanAtLevel:(UInt16)levelNum forStream:(mcWMStream*)aStream;
- (enum mcPlanningStatus) generateProbabilisticHighLevelPlansForStream:(mcWMStream *)aStream;
- (enum mcPlanningStatus) generatePlansUsingAbstractPlanStackForStream:(mcWMStream*)aStream;
- (void) generateOrdinalFormForPlanSet;
- (NSMutableArray*) createMotiveArrayFromNodeEdgeArray:(NSMutableArray *)nodeEdges;
- (enum mcPlanningStatus) createPlanAtLevelNumber:(UInt16)levelNum forStream:(mcWMStream *)aStream counter:(UInt16)count;
- (enum mcPlanningStatus) createPredictivePlanAtLevelNumber:(UInt16)levelNum forStream:(__unsafe_unretained mcWMStream *)aStream counter :(UInt16)count;
- (enum mcPlanningStatus) inferSuperLevelPlanFromPlanAtLevel:(UInt16)levelNum forStream:(mcWMStream*)aStream;
- (enum mcPlanningStatus) updateL3PlanBasedOnL2Inference:(mcCMEdge*)L2Inference forStream:(mcWMStream*)aStream;
- (enum mcPlanningStatus) updateConnectionsBetweenPlanLevelsForStream:(mcWMStream *)aStream;
- (enum mcPlanningStatus) defineMotivesAndL1SegmentsForL2PlanGivenStream:(mcWMStream*)aStream;
- (void) filterSelectionSet:(NSMutableArray*)selectionSet toMatchPlanningTierForMotive:(mcCMMotive*)aMotive;
- (void) filterSelectionSet:(NSMutableArray *)selectionSet toMatchPlanningTierForMotiveBoundary:(mcCMEdge*)motiveBoundary;
- (void) filterSelectionSet:(NSMutableArray *)selectionSet toTier:(UInt8)tier;
- (NSUInteger) identifyLevelNeedingPlanForStream:(mcWMStream*)aStream;
- (void) createSetOfPrimedL2PhraseBoundariesGivenL2Plan:(NSMutableArray *)L2Plan;
- (mcCMMotive*) popNextL2MotiveFromPlanForStream:(mcWMStream *)aStream counter:(UInt64)iterCount;
- (id) initWithMusiCog:(mcMusiCogController*)mc;

@end
