//
//  mcWMSegment.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-08.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>

@class mcWMStream, mcEvent, mcCMMotive, mcCMNode, mcCMEdge;

@interface mcWMSegment : NSObject <NSCopying>

@property (strong, nonatomic) NSMutableArray* elements; // all events or chunks in the segments
@property (weak, nonatomic) mcWMStream* stream;         // a pointer to the stream that contains this segment
@property (assign, nonatomic) UInt16 level;             // the nesting level of this segment
@property (strong, nonatomic) mcEvent* boundaryEvent;   // the first event in the segment
@property (strong, nonatomic) mcCMMotive* motive;       // motive for this segment
@property (assign, nonatomic) float pitchRecognition;   // for pitch CM: 0.3 = schema, 0.8 = invariance, 1.0 = identity
@property (assign, nonatomic) float rhythmRecognition;  // for rhythm CM: 0.3 = schema, 0.8 = invariance, 1.0 = identity
@property (assign, nonatomic) float cognitiveSalience;  // salience used to retain in WM
@property (assign, nonatomic) float habituation;        // habituation value
@property (assign, nonatomic) float modeConfidence;     // high = diatonic, low = chromatic
@property (assign, nonatomic) BOOL isChunk;             // a flag indicating whether this segment has been chunked
@property (assign, nonatomic) BOOL isStart;             // yes if this is the first segment formed
@property (assign, nonatomic) BOOL isClosed;            // yes if this segment is completed
@property (assign, nonatomic) BOOL isCounted;           // yes if edge counts updated while in WM

+ (void) flattenSegment:(mcWMSegment*)aSegment toArrayOfEvents:(NSMutableArray*)anArray copyItems:(BOOL)flag;
+ (void) flattenSegment:(mcWMSegment*)aSegment toArrayOfSegments:(NSMutableArray*)anArray;
+ (void) flattenSegment:(mcWMSegment *)aSegment toArrayOfSegments:(NSMutableArray *)anArray atLevel:(UInt16)levelNum;

- (BOOL) isNoteSegment;
- (mcWMSegment*) getLastSegment;
- (mcEvent*) lastEvent;
- (void) extendWithEvent:(mcEvent*)anEvent;
- (void) extendByInsertingEvent:(mcEvent*)anEvent;
- (float) recognition;
- (void) updateCognitiveSalienceWithElement:(/*__unsafe_unretained*/ mcWMSegment*)aSegment andTime:(long)timestamp;
- (void) updateHabituationForAge:(double)age cognitiveSalience:(float)salience andThreshold:(float)thresh atTime:(float)timestamp;
- (void) setTerminal:(/*__unsafe_unretained*/ mcCMEdge*)aTerminal;
- (void) setRecognitionWithInference:(/*__unsafe_unretained*/ mcCMNode*)inference;
- (float) getMeanCohesion;
- (BOOL) hasMatchingElements:(id)object;
- (id) initWithElement:(id)anElement;
- (id) initWithStream:(mcWMStream*)aStream;
- (UInt16) count;

@end

/****
 
 NOTE: As a convenience, we're going to store the inferred edge sequence in the motive's pitchBranch/rhythmBranch.
 Once the segment is "closed," we'll increment all the edge counts in both branches. Voila! Counts updated!
 
 **/
