//
//  mcWMStream.h
//  MusiCog
//
//  Created by James Maxwell on 2013-06-08.
//  Copyright (c) 2013 James Maxwell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>

@class mcMusiCogController, mcCMState, mcWMSegment, mcCMNode, mcCMEdge, mcEvent, RunningStat;

@interface mcWMStream : NSObject<NSCopying>

@property (weak, nonatomic) mcMusiCogController* musiCog;   // the master controller class
@property (assign, nonatomic) UInt32 streamID;              // index of the stream in activeStreams
@property (strong, nonatomic) NSMutableArray* segments;     // a stack with all segments (zero is most recent)
@property (assign, nonatomic) UInt8 capacity;               // max elements before stream starts trying to discard
@property (assign, nonatomic) UInt8 mode;                   // estimated Greek mode
@property (assign, nonatomic) UInt8 tonality;               // estimated tonality (as pitch-class)
@property (assign, nonatomic) float modeConfidence;         // indicates chromaticism: low confidence = chromatic, high confidence = diatonic (modal)
@property (assign, nonatomic) float *chroma;                // chroma vector for this stream
@property (assign, nonatomic) BOOL isResting;               // whether the stream has onsets/sustains in the current group
@property (assign, nonatomic) BOOL hasNewEvent;             // yes if this stream has received an event in the last iteration
@property (assign, nonatomic) UInt64 endTime;               // ms time of the end of the stream (including event duration)
@property (strong, nonatomic) mcCMState* pitchState;        // state of the pitch model
@property (strong, nonatomic) mcCMState* rhythmState;       // state of the rhythm model
@property (strong, nonatomic) NSArray* previousStates;      // [pitchState, rhythmState] from previous timestep
@property (assign, nonatomic) BOOL generateOnNextInput;     // a flag to control continuations
@property (assign, nonatomic) enum mcPlanningStatus planningStatus; // indicates whether planner succeeded in generating a plan (or error code)
@property (strong, nonatomic) RunningStat* rsCohesion;      // running stddev of cohesion

+ (void) flattenStream:(mcWMStream*)aStream
       toArrayOfEvents:(NSMutableArray*)anArray
             copyItems:(BOOL)flag;
+ (void) flattenStream:(mcWMStream *)aStream
 toArrayOfNoteSegments:(NSMutableArray*)segments;
+ (void) flattenStream:(mcWMStream *)aStream
extractingAllChunksToArray:(NSMutableArray *)allChunks;

- (void) resetStates;
- (UInt32) count;
- (BOOL) segmentArray:(NSArray*)anArray containsSegmentMatchingSegment:(mcWMSegment*)aSegment;
- (float) meanPitch;
- (float) meanCognitiveSalience;
- (mcWMSegment*) mostRecentNoteSegment;
- (mcWMSegment*) segmentPrecedingSegment:(/*__unsafe_unretained*/ mcWMSegment*)aSegment;
- (NSArray*) getRecentEvents:(UInt16)count;
- (mcWMSegment*) segmentForTerminal:(mcCMEdge*)aTerminal;
- (void) addEvent:(mcEvent*)anEvent;
- (BOOL) handleSingletonSegments;
- (void) updateStreamContents;
- (void) updateStateWithInference:(/*__unsafe_unretained*/ mcCMNode*)inference;
- (mcWMSegment*) segmentAtLevelPrecedingChunk:(mcWMSegment*)aChunk;
- (BOOL) replaceSegment:(/*__unsafe_unretained*/ mcWMSegment*)aSegment withChunk:(/*__unsafe_unretained*/ mcWMSegment*)aChunk atIndex:(NSUInteger)anIndex;
- (BOOL) statesChangedFromPreviousTimestep;
- (void) getEighthGrid:(float*)aGrid forType:(UInt8)typeFlag accumulate:(BOOL)flag; // 0 = duration, 1 = accentStrength, accumalate YES adds to vector

// Some thread-safe array operations for the stream's contents
- (void) removeSegment:(mcWMSegment*)aSegment;                                  // add segment to self.segments
- (void) removeSegmentAtIndex:(NSUInteger)anIndex;                              // remove the segment at a given index of self.segments
- (void) insertSegment:(mcWMSegment*)aSegment atIndex:(NSUInteger)anIndex;      // insert segment to self.segments at index
- (void) addSegment:(mcWMSegment*)aSegment;                                     // add segment to end of self.segments
- (void) replaceSegment:(mcWMSegment*)aSegment atIndex:(NSUInteger)anIndex;     // replace segment in self.segments at index
- (void) removeSegmentsInArray:(NSArray*)anArray;                               // given an array of segments, remove them all from self.segments

- (id) initWithMusiCog:(mcMusiCogController*)mc;

@end
