/*
 *  vectorUtils.h
 *  ManuScore
 *
 *  Created by James Maxwell on 09-11-03.
 *  Copyright 2009 Rubato-Music.com. All rights reserved.
 *
 */

#include "stddef.h"

float logAdd(float x, float y);

void vectorMultiply(float* vectorA, float* vectorB, float* productVector, int vectorLength);

float absoluteRatio(float valA, float valB);

void vectorSum(float* aVector, float* sum, int vectorLength);

void intVectorSubtract(int* vectorA, int* vectorB, int* differenceVector, int vectorLength);

void intVectorSum(int* aVector, int* sum, int vectorLength);

// Note: This stores the result of the member-for-member sum in vectorA
void sumOfVectors(float* vectorA, float* vectorB, int vectorLength);

void vectorNormalize(float* vector, int vectorLength);

void vectorVNormalize(float* vector, int vectorLength);

void vectorNormalizeToMax(float* vector, int vectorLength);

void longVectorNormalize(long* longVector, float* normVector, long vectorLength);

float vectorMean(float* aVector, int vectorLength);

float nonZeroVectorMean(float* aVector, int vectorLength);

float vectorStdDev(float* aVector, int vectorLength);

float vectorVariance(float* aVector, int length);

float nonZeroVectorStdDev(float* aVector, int vectorLength);

float euclideanDistanceFromVectorAtoB(float* vectorA, float* vectorB, int vectorLength);

float euclideanDistanceFromIntVectorAtoB(int* vectorA, int* vectorB, int vectorLength);

float hammingDistanceFromVectorAtoB(float* vectorA, float* vectorB, int vectorLength);

int intHammingDistanceFromVectorAtoB(long* vectorA, long* vectorB, int vectorLength);

float pitchDistanceFromVectorAtoB(float* vectorA, float* vectorB, int vectorLength);

float rhythmicDistanceFromAtoB(float* vectorA, float* vectorB);	// length is always 2: ED, duration

float vectorMaxValue(float* aVector, int vectorLength);

int vectorMaxIndex(float* aVector, int vectorLength);

float vectorMinValue(float* aVector, int vectorLength);

int vectorMinIndex(float* aVector, int vectorLength);

int intVectorMaxIndex(int* aVector, int vectorLength);

int vectorPeakCount(float* aVector, int vectorLength);

int intSequenceLength(int* aSequence, int maxSeqLength);

int vectorIdentity(float* vectorA, float* vectorB, int vectorLength);

int vectorPeakIdentity(float* vectorA, float* vectorB, int vectorLength);

int intVectorIdentity(int* vectorA, int* vectorB, int vectorLength);

int intVectorIdentityWithPassiveFlags(int* vectorA, int* vectorB, int* isPassive, int vectorLength);

int intVectorMatchCount(int* vectorA, int* vectorB, int vectorLenth);

int sequenceIdentity(int* vectorA, int* vectorB, int vectorLength);

int lengthOfLongestSubSequence(int* storedSequence, int* newSequence, int sLen, int nLen);

void lengthAndEndOfLongestSubSequence(int* storedSequence, int* newSequence, int sLen, int nLen, int* subseqEnd, int* subseqLen);

int sequenceContainsCoincidence(int* seqRow, int seqLength, int coinc);

void scaleForZeroValuedBeliefs(float* vector, int vectorLength);

void vectorRotateLeft(long* aVector, long* buffer, long vectorLength, int steps);

void fvectorRotateLeft(float* aVector, float* buffer, int vectorLength, int steps);

void vectorRotateRight(long* aVector, long* buffer, int vectorLength, int steps);

void vectorRotateRightTruncate(int* aVector, int* buffer, int vectorLength, int steps);

float combinePriorMeanWithCurrentMeanForLength(float priorMean, float currentMean, int length);

float calculateEntropyForWeightsVector(float *weights, int length);

float normEntropyForWeightsVector(float *weights, int length);

void vectorScale(float* aVector, int length);

size_t getDigits(int num);

// A few fuzzy membership functions

float membershipWithTriangle(float s, float p, float x);    // start, peak, input (x)

float membershipTriangleWithPeakAndWidth(float p, float w, float x);

float membershipWithInverseTriangle(float s, float p, float x);    // start, peak, input (x)

float membershipWithRamp(float min, float max, float x);    // min, max, input (x)

float proportionalWidthTriangularWindowWithPeakAndFactor(float peak, float factor, float x);

int probabilisticChoiceOfNonZeroIndexFromWeightsVector(float* weights, int length);

int probabilisticChoiceOfIndexFromLongWeightsVector(long* weights, int offset, int length);

int randNum(int lim);

float roundUp(float toRound);

long greatestCommonDivisorOfXY(long x, long y);

long greatestCommonDivisorOfArray(long* anArray, int length);

float vectorAutocorrelation(float* aVector, int lag, int length);